#include <stdio.h>
#include <string.h>
#include <math.h>

#include "SimpleMOCinput.logic.h"

/*****************************
   User editing starts here
******************************/
extern Utils *g_pUtils;
RCHANDLE g_cedl;
static char g_exceptionBuffer[8192];

#ifdef _WIN32
#define SNPRINTF _snprintf
#else
#define SNPRINTF snprintf
#endif

RCINT64 GetMinorVersionProduced()
{
    return 10; /* version 3.10 */
}

char* ConvertToTAP3DateTime(struct tm *pTimeData, char *pBuf, int maxLen)
{
    SNPRINTF(pBuf, maxLen, "%04d%02d%02d%02d%02d%02d",
        pTimeData->tm_year + 1900,
        pTimeData->tm_mon + 1,
        pTimeData->tm_mday,
        pTimeData->tm_hour,
        pTimeData->tm_min,
        pTimeData->tm_sec);
    return pBuf;
}

#define CONTROLLED_RUN(x)                                                                           \
        {                                                                                           \
            int result = x;                                                                         \
            if (0 != result) {                                                                      \
                SNPRINTF(g_exceptionBuffer, sizeof(g_exceptionBuffer), "Failed to execute:\n%s\n(result:%d)\n", #x, result); \
                g_pUtils->pfnThrowException(g_exceptionBuffer);                                     \
                return;                                                                             \
            }                                                                                       \
        }

void ProcessHEADER(RCHANDLE treeRoot, HEADER *pHEADER)
{
    XMLHANDLE tbXml;
    RCHANDLE tbNode;
    static char tmp[4096];

    CONTROLLED_RUN( g_pUtils->pfnCompileSubTreeFromXmlTemplate("MainBody.xml", &tbXml) )
    CONTROLLED_RUN( g_pUtils->pfnAddSubTree(treeRoot, tbXml, &tbNode) )

    CONTROLLED_RUN( g_pUtils->pfnCreateChildVariable(tbNode, "callEventDetails", &g_cedl) )

    CONTROLLED_RUN( g_pUtils->pfnSetString( tbNode, "batchControlInfo.sender", pHEADER->SENDER, strlen(pHEADER->SENDER)) )
    CONTROLLED_RUN( g_pUtils->pfnSetString( tbNode, "batchControlInfo.recipient", pHEADER->RECIPIENT, strlen(pHEADER->RECIPIENT)) )
    CONTROLLED_RUN( g_pUtils->pfnSetString( tbNode, "batchControlInfo.fileSequenceNumber", pHEADER->FSN, strlen(pHEADER->FSN)) )

    ConvertToTAP3DateTime(&pHEADER->FILE_CREATION_DATE, tmp, sizeof(tmp));
    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            tbNode, "batchControlInfo.fileCreationTimeStamp.localTimeStamp",
	    tmp, strlen(tmp)) )
    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            tbNode, "batchControlInfo.fileCreationTimeStamp.utcTimeOffset", "+0000", 5) )

    ConvertToTAP3DateTime(&pHEADER->FILE_CREATION_DATE, tmp, sizeof(tmp));
    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            tbNode, "batchControlInfo.transferCutOffTimeStamp.localTimeStamp",
            tmp, strlen(tmp)) )
    CONTROLLED_RUN(
        g_pUtils->pfnSetString( tbNode, "batchControlInfo.transferCutOffTimeStamp.utcTimeOffset", "+0000", 5) )

    ConvertToTAP3DateTime(&pHEADER->FILE_CREATION_DATE, tmp, sizeof(tmp));
    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            tbNode, "batchControlInfo.fileAvailableTimeStamp.localTimeStamp",
            tmp, strlen(tmp)) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString( tbNode, "batchControlInfo.fileAvailableTimeStamp.utcTimeOffset", "+0000", 5) )
}

void ProcessMOC(RCHANDLE treeRoot, MOC *pMOC)
{
    XMLHANDLE mocXml;
    RCHANDLE mocNode, utcCode, recEntityCode, exchangeRateCode;
    static char tmp[4096];
    double charge;

    CONTROLLED_RUN( g_pUtils->pfnCompileSubTreeFromXmlTemplate("SimpleMOCtemplate.xml", &mocXml) )
    CONTROLLED_RUN( g_pUtils->pfnAddSubTree(g_cedl, mocXml, &mocNode) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString( 
	    mocNode, "mobileOriginatedCall.basicCallInformation.chargeableSubscriber.simChargeableSubscriber.imsi", pMOC->IMSI, strlen(pMOC->IMSI)) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString( mocNode, "mobileOriginatedCall.basicCallInformation.destination.calledNumber", pMOC->CALLED_NUMBER, strlen(pMOC->CALLED_NUMBER)) )

    ConvertToTAP3DateTime(&pMOC->CALL_DATE_TIME, tmp, sizeof(tmp));
    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            mocNode, "mobileOriginatedCall.basicCallInformation.callEventStartTimeStamp.localTimeStamp", 
            tmp, strlen(tmp)) )
    CONTROLLED_RUN(
        g_pUtils->pfnNavigateAndObtainHandle(
            mocNode, "mobileOriginatedCall.basicCallInformation.callEventStartTimeStamp.utcTimeOffsetCode", &utcCode) )
    CONTROLLED_RUN(
        g_pUtils->pfn_get_utcTimeOffsetCode(treeRoot, "+0000", utcCode) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetInteger(
            mocNode, "mobileOriginatedCall.basicCallInformation.totalCallEventDuration", 
            pMOC->DURATION) )

    CONTROLLED_RUN(
        g_pUtils->pfnNavigateAndObtainHandle(
            mocNode, "mobileOriginatedCall.locationInformation.networkLocation.recEntityCode", &recEntityCode) )
    CONTROLLED_RUN(
        g_pUtils->pfn_get_recEntityCode(treeRoot, 1, pMOC->MSCID, NULL, NULL,  NULL,  NULL,  NULL, recEntityCode) )

    CONTROLLED_RUN(
        g_pUtils->pfnNavigateAndObtainHandle(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].exchangeRateCode", &exchangeRateCode) )
    CONTROLLED_RUN(
        g_pUtils->pfn_get_exchangeRateCode(
            treeRoot,
            (RCINT64)5,
            (RCINT64) (0.5 + 100000.0*pMOC->EXCHANGE_RATE),
            exchangeRateCode) )

    charge = pMOC->CHARGE;
    CONTROLLED_RUN(
        g_pUtils->pfnSetInteger(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].chargeDetailList.[0].charge", (RCINT64)(0.5 + 1000.0*charge)) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].chargeDetailList.[0].dayCategory", "N", 1) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].chargeDetailList.[0].timeBand", "I", 1) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetInteger(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].callTypeGroup.callTypeLevel1", pMOC->NATIONAL_IND) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].callTypeGroup.calledCountryCode", 
	    pMOC->CALLEDCOUNTRYCODE, strlen(pMOC->CALLEDCOUNTRYCODE)) )

    CONTROLLED_RUN(
        g_pUtils->pfnSetString(
            mocNode, "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].chargedItem", "D", 1) )
    CONTROLLED_RUN(
        g_pUtils->pfnSetInteger(
            mocNode,
            "mobileOriginatedCall.basicServiceUsedList.[0].chargeInformationList.[0].chargeDetailList.[0].chargeableUnits",
            pMOC->CHARGEABLE_UNITS))
}

/*****************************
   User editing stops here
 ****************************/
